import numpy

def euclidean(a: numpy.ndarray, b: numpy.ndarray) -> float:
    """Compute the Euclidean distance between two vectors."""

def sqeuclidean(a: numpy.ndarray, b: numpy.ndarray) -> float:
    """Compute the squared Euclidean distance between two vectors."""

def cosine(a: numpy.ndarray, b: numpy.ndarray) -> float:
    """Compute the cosine similarity between two vectors."""

def cdist(a: numpy.ndarray, b: numpy.ndarray, metric: str) -> numpy.ndarray:
    """Compute the distance between each pair of the two collections of vectors."""

def pdist(a: numpy.ndarray, metric: str) -> numpy.ndarray:
    """Compute the distance between each pair of the collection of vectors."""
